import React, { useContext, useState, useMemo, useEffect, createContext } from 'react';

const Context = createContext();
const map = new Map();
const prevState = new Map();
let index = 0;

export function useRedux(mapState) {
  const store = useContext(Context);
  const mappedState = useMemo(() => mapState(store.getState()), [store]);
  const [state, setState] = useState(mappedState);
  useEffect(() => {
    const i = index++;
    map.set(i, {
      setState,
      mapState,
    });
    return () => {
      map.delete(i);
      prevState.delete(i);
    };
  }, [store]);
  return state;
}

export function useDispatch(mapDispatch, watch) {
  const store = useContext(Context);
  return useMemo(() => mapDispatch(store.dispatch), watch ? [...watch, store] : undefined);
}

export function ReduxProvider({ store, children }) {
  useEffect(() => {
    const unsubscribe = store.subscribe(() => {
      const state = store.getState();
      map.forEach(({ setState, mapState }, i) => {
        const mappedState = mapState(state);
        if (mappedState !== prevState.get(i)) {
          prevState.set(i, mappedState);
          setState(mappedState);
        }
      });
      return () => unsubscribe();
    });
  }, [store]);

  return <Context.Provider value={store}>{children}</Context.Provider>
}
